/* DIMMER VERSION 1.1                             */
/*                                                */
/*                __    _                         */                     
/*     __________/ /_  (_)________   ___  __  __  */ 
/*    / ___/ ___/ __ \/ / ___/ __ \ / _ \/ / / /  */
/*   (__  ) /__/ / / / / /  / /_/ //  __/ /_/ /   */
/*  /____/\___/_/ /_/_/_/  / .___(_)___/\__,_/    */
/*                        /_/                     */
/*                                                */
/* (c) C.Schirp  released for non-commercial use  */
/* IR-routines copyright by Jos van Eijndhoven    */
/*                                                */
/* V1.0  Erstausgabe                              */
/* V1.1  EVG-Lichtstromkorrektur                  */
/*                                                */


#include <d:\projekte\picsel\cc5\16F628.h>
#include <d:\projekte\picsel\cc5\int16CXX.h>

#pragma config = 0x3F18
// Standard ohne ext reset 0x3F18
// Standard mit ext Reset  0x3F38 

#define HARD_PWM

/* Defines */

#define INPUT 1
#define OUTPUT 0

#define MIN_DIM 1	/* minimaler Dimgrad */
#define MAX_DIM 10	/* maximaler Dimgrad */

#define SAVETIME	625	/* nach SAVETIME mal 8ms wird der dimwert ins eeprom geschrieben */

unsigned char dim_grade;	 //debounced and inverted dimming grade 10...100:
unsigned char dim_1;
unsigned char dim_2;
//const unsigned char dtable[11] = {0, 26, 34, 43, 56, 72, 92,119,154,198,255}; // tabelle aus microcontroller.net/LED-Fading
const unsigned char dtable[11] = {0, 64, 73, 82, 95,108,125,146,174,210,255}; // korrektur LED-Fading und Quicktronic-Lichtstrom-Graph
unsigned char duty;

uns16  ircode;
uns16  save_counter;
unsigned char irled; // 1 if light is on
unsigned char old_dim_grade;


unsigned char tmr0cpy;



//bit Monitor; // Shadow fuer PB6

bit irledn     @ PORTB.0;

/*------------------------------------------------------------------*/
/*                                                                  */
/*                                                                  */
/*                                                                  */
/*                                                                  */
/*                                                                  */
/*------------------------------------------------------------------*/


#pragma origin 4
interrupt int_server( void)
{
    int_save_registers    // W, STATUS (and PCLATH)

    if ( INTF)
    {
        /* RB0/INT interrupt */
        irled.0 = !irledn;
        INTF = 0;
        if (irled)     // request interrupt on next edge (up or down)
           INTEDG = 1; // if 1, then interrupt on rising edge
        else
           INTEDG = 0; // is part of OPTION_REG, is 1 on reset

        /* finish and restart TMR0 */
        tmr0cpy = TMR0;
        TMR0 = 0;
        T0IF = 1; // end polling wait-loop, similar to TRM0 wraparound
    }

   /*
     NOTE: GIE is AUTOMATICALLY cleared on interrupt entry and set
           to 1 on exit (by RETFIE). Setting GIE to 1 inside the
           interrupt service routine will cause nested interrupts
           if an interrupt is pending. Too deep nesting may crash
           the program !
   */

    int_restore_registers // W, STATUS (and PCLATH)
}

/*------------------------------------------------------------------*/
/*   II  N   N  II TTTTT                                            */
/*   II  NN  N  II   T                                              */
/*   II  N N N  II   T                                              */
/*   II  N  NN  II   T                                              */
/*   II  N   N  II   T                                              */
/*------------------------------------------------------------------*/


void init ()
{
	
	
	/* PORT A Config */
	CMCON = 0x07;		// kein Komparator

	TRISB = 0b0000.0001; 	// Alles Ausgnge - nur RB0/Int Eingang
	TRISA = 0b0000.0000;	// alles Output
	
	PORTB = 0b0000.0000;	// HINT: erst Direction, dann Wert!
	PORTA = 0x00;	


	/*sonstiges*/
	OPTION_REG = 0x84;/* pull-up portB, falling edge int, no WDT prescaler, prescale=32 */
	
#ifdef HARD_PWM
	/* Timer 1 und Timer 2 fr PWM konfigurieren */
	PR2 = 0xFF;		// Periode: 255 Takte
	CCPR1L = 0;		// Duty = 0
	T2CON = 0x05;		// Timer 2 = 0n, Prescaler = 4   (-> 4MHz/4/1024/4 => ca. 244Hz PWM-Grundfrequenz)	
	CCP1CON=12;		// Timer1 im PWM-mode  (8-Bit PWM, LSBs des Duty Cycle nicht benutzt)
#endif



	/* Variablen initialisieren */
	dim_grade = MIN_DIM;	// 10% minimum
	save_counter = 0; 	// nix schreiben
	
	irled = 	0;


	INTCON = 0; 			// reset and disable all interrupts by default
	INTE = 	1;
	GIE = 	 1;				// Enable Interrupts
	

}

/*------------------------------------------------------------------*/
/*    EEEE  EEEE  PPPP   RRRR    OOO   M     M                      */
/*    E     E     P   P  R   R  O   O  MM   MM                      */
/*    EEE   EEE   PPPP   RRRR   O   O  M M M M                      */
/*    E     E     P      R R    O   O  M  M  M                      */
/*    EEEE  EEEE  P      R  R    OOO   M     M                      */
/*------------------------------------------------------------------*/



void write_eeprom( unsigned char adr, unsigned char data)
{
	bit prev_gie = GIE;
	GIE = 0; 		// disable interrupts
	EEIF = 0; 		// clear eprom interrupt flag

	while (WR)
		; 			// wait if previous write is still busy

	EEADR = adr; 	// valid range is from 00h to 7Fh (128 bytes)
	EEADR &= 0x7f; 	// prevent bugs to overwrite program memory
	EEDATA = data;
	WREN = 1;
	EECON2 = 0x55;
	EECON2 = 0xAA;
	WR = 1; 		// start the actual write

	WREN = 0; 		// protect against unintended writes
	GIE = prev_gie; // restore interrupts
}

unsigned char read_eeprom( unsigned char adr)
{
	EEADR = adr; 	// valid range is from 00h to 7Fh (128 bytes)
	EEADR &= 0x7f; 	// prevent bugs to exceed program memory
	RD = 1;
	return EEDATA;
}

/*------------------------------------------------------------------*/
/*    K  K  EEEE  Y   Y   SSS                                       */
/*    K K   E      Y Y   S                                          */
/*    KK    EE      Y     SSS                                       */
/*    K K   E       Y        S                                      */
/*    K  K  EEEE    Y     SSS                                       */
/*------------------------------------------------------------------*/


void restore_state(void)
{
	unsigned char tester;

	dim_grade = read_eeprom(0);
	dim_1 = read_eeprom(1);
	dim_2 = read_eeprom(2);

	// 2 aus 3
	if (dim_grade==dim_1)
	{
	}
	else if (dim_grade==dim_2)
	{
	}
	else if (dim_1 == dim_2)
	{
		dim_grade = dim_1;
	}

	// Gueltigkeit pruefen
	if (dim_grade < MIN_DIM)
	{
		dim_grade = MIN_DIM;
	}
	else if (dim_grade > MAX_DIM)
	{
		dim_grade = MAX_DIM;
	}
	
	old_dim_grade = dim_grade;	// alten dimgrad setzen, damit er nicht ins EEPROM geschrieben wird
	
	duty = dtable[dim_grade];
			
	CCPR1L = duty;

}

/*------------------------------------------------------------------*/
/*    RRR    CCC    55555                                           */
/*    R  R  C   C   5                                               */
/*    RRR   C       5555                                            */
/*    R R   C   C       5                                           */
/*    R  R   CCC    5555                                            */
/*------------------------------------------------------------------*/

// 4MHz crystal, 1MHz cycle time, TMR0 prescale=32: timer units of 32 usec.
// TRM0 expires at 8 msec.

extern uns8 tmr0wait( void)
{
	tmr0cpy = 0xff; // potential longest waiting time, reduced by irled
	while (!T0IF) // wait for TMR0 wrap-around or IRled
		;

	T0IF = 0;
	return tmr0cpy;
}

void ir_bogus( void)
{
	// just eat irled input, as long as we have fast pulses
	uns8 t;

	t = 0;
	while (t < 200)
	   t = tmr0wait();

	ircode = 0xffffL; // bogus code;
}

void ir_philips_rc5(void) // now assuming no 'extended' RC5, so S2==1
{
	// 1 is sequence dark,light:  2x 900us == 2x28
	// 0 is sequence light,dark:  2x 900us == 2x28
	// frame is: one,one,toggle,11xdata
	unsigned char t;
	unsigned char i;

	t = 0;
	i = 0;
	ircode = 0L;
	// starting now in the 2nd (light) half S2
	while (t < 80)
	{
	   // last half of current bit still to be measured, its timer is running
	   ircode <<=1; // insert current value
	   if (irled) // irled input is active low
		   ircode.0 = 1;
	   i++;

	   t = tmr0wait();

	   if (t < 38) // new codebit is same as current, edge at bit boundary
	   {
	   	  t=tmr0wait(); // skip first half of new bit
	   }
	}

	ircode.high8 &= 0x07; // clear start bits S2 and T

	if (i < 13)
	{
		ircode = 0xffffL; // bogus
	}
}


void get_ir( void)
{
	unsigned char ton;
	unsigned char toff;

    ton = tmr0wait(); // measure first light
	toff = tmr0wait(); // following first dark

    if (ton  > 20 && ton  < 36 &&  // nominal 900/32 = 28
        toff > 20 && toff < 36)  // nominal 900/32 = 28
    {
		ir_philips_rc5();
	}
	else
	{
		ir_bogus();
	}
}

void do_remote(void)
{
	unsigned char h;



	// systemadresse testen
	h = ircode.high8 & 0b.0000.0111;
	if (h != 4)
	{
		return;
	}

	h = ircode.low8 & 0b.1100.0000;
	if (h != 0)
	{
		return;
	}

	// okay, Kommando gilt uns: Kommando extrahieren
	h= ircode.low8 & 0b.0011.1111;

	// Quellenwahltasten = 1..8
	if (h<=9)
	{
	
		dim_grade = h;

	}
	
	// Sonderfall 0/10
	if (h == 0)
	{
		dim_grade = 10;
	}

	if (h == 16) // Volume +
	{
		if (dim_grade <= 9)
		{
			dim_grade++;
		}
	}
	else if (h == 17) // Volume -
	{
		if (dim_grade > 1)
		{
			dim_grade--;
		}
		
	}


}


/*------------------------------------------------------------------*/
/*     M     M    A    II  N   N                                    */
/*     MM   MM   A A   II  NN  N                                    */
/*     M M M M  A   A  II  N N N                                    */
/*     M  M  M  AAAAA  II  N  NN                                    */
/*     M     M  A   A  II  N   N                                    */
/*------------------------------------------------------------------*/

void main (void)
{
	unsigned char tmp;

	init();

	restore_state();

	
	//CLRWDT;
	//clrwdt();

	while (1)
	{


		tmp = tmr0wait(); // wrap-around or interrupt, wrap-around takes 8ms
	
		//pause(10);

	    	if (irled) //came out of wait by interrupt: light on, start of new code
		{
			get_ir();
			do_remote();
		}
		
		// wenn sich der dimmgrad gendert hat, schreiben
		if (old_dim_grade != dim_grade)
		{
			save_counter = SAVETIME;
			
			old_dim_grade = dim_grade;
			
			duty = dtable[dim_grade];
			
			CCPR1L = duty;
			
		}
		
		// wartezeit vor dem speichern abgelaufen?
		if (save_counter == 1)
		{
		
			write_eeprom(0,dim_grade);
			write_eeprom(1,dim_grade);
			write_eeprom(2,dim_grade);
			
			save_counter = 0; // speichern erledigt
		}
		else if (save_counter > 1)
		{
			save_counter--;
		}
		
		//clrwdt();

	} /* while (1) */
}